@@ -19,6 +19,7 @@ import shutil
 import inspect
 from pprint import pprint
 from random import randint
+from copy import copy
 
 import myhdl
 
@@ -79,19 +80,44 @@ class _fpga(object):
 
         return os.path.isdir(pth)
 
-    def convert(self, name=None, to='verilog', **kwargs):
-        """ convert the myhdl design to Verilog or VHDL and copy to working dir
-        """
-        assert to in ('verilog', 'vhdl'), \
-               "Incorrect conversion type: %s" % (self.conversion_type)
-
+    def _get_name(self, name=None):
         if name is None:
             if self.top_name is not None:
                 name = self.top_name
             else:
                 name = self.top.func_name
 
-        # get the top-leve ports and parameters.
+        return name
+
+    def _scrub(self, filelist):
+        """
+        remove the generated files from the file list, if the
+        toolflow is run subsequent times the converted files 
+        are added each time (especially bad when running one
+        conversion type and then the other)
+        """
+        print(filelist)
+        name = self._get_name()
+        fl = copy(filelist)
+        for ii,ff in enumerate(fl):
+            print(" ... %s"%(ff))
+            if (ff == "%s.v"%(name) or 
+                ff == "%s.vhd"%(name)):
+                print("scrubbing %s"%(ff))
+                filelist.remove(ff)
+            elif "pck_myhdl" in ff:
+                print("scrubbing %s"%(ff))
+                filelist.remove(ff)
+
+    def convert(self, name=None, to='verilog', **kwargs):
+        """ convert the myhdl design to Verilog or VHDL and copy to working dir
+        """
+        assert to in ('verilog', 'vhdl'), \
+               "Incorrect conversion type: %s" % (self.conversion_type)
+
+        name = self._get_name(name)
+
+        # get the top-level ports and parameters.
         pp = inspect.getargspec(self.top) # pp = portsparams
 
         # all of the arguments (no default values) are treated as
@@ -155,6 +181,8 @@ class _fpga(object):
             myhdl.toVHDL(self.top, **pp)
             self.name = name
             self.vfn = "%s.vhd"%(name)
+        else:
+            raise ValueError("Incorrect conversion target %s"%(to))
 
         # make sure the working directory exists
         assert self.pathexist(self._path)
@@ -162,8 +190,9 @@ class _fpga(object):
         # copy files etc to the working directory
         tbfn = 'tb_'+self.vfn
         ver = myhdl.__version__
-        ver = ver.replace('.','')
-        ver = ver.replace('-dev','')
+        # remove special characters from the version
+        for sp in ('.', '-', 'dev'):
+            ver = ver.replace(sp,'')
         pckfn = 'pck_myhdl_%s.vhd'%(ver)
         for src in (self.vfn,tbfn,pckfn):
             dst = os.path.join(self._path,src)
@@ -175,11 +204,45 @@ class _fpga(object):
                 print('   moving %s --> %s'%(src,self._path))
                 shutil.move(src,self._path)
 
+        if to.lower() == 'verilog':
+            filelist = (self.vfn,)
+        elif to.lower() == 'vhdl':
+            filelist = (self.vfn, pckfn,)
+
+        return filelist
+
     def add_port(self, name, sig, pins, **kwargs):
-        assert isinstance(pins, (list,tuple)), "pins must be a list/tuple of pins"
+        if isinstance(pins, (str, int)):
+            pins = (pins,)
+        assert isinstance(pins, (list,tuple)), \
+            "pins must be a list/tuple of pins (or a single str/int)"
         self.ports[name] =  _port(name, sig, pins, **kwargs)
         if isinstance(sig, Clock):
             self.clocks[name] = self.ports[name]
+
+    def name_port(self, name, port, slice_=None, **kwargs):
+        """Extract "slices" from a port
+        Makes it possible to extract and name a single port (a range 
+        of ports) out of a larger port array. This could be useful to 
+        give meaningful names to single ports e.g. on Papilio Wings.
+
+        Example: board.name_port('led', 'wingC', 7)
+        where wingC is a 16-bit port array.
+
+        To extract a port range the slice class has to be used. E.g.
+        board.name_port('MSB', 'wingC', slice(16,8))"""
+
+        p = self.ports[port]
+        if slice_ is None:
+            pins = p.pins
+            sig = p.sig
+        else:
+            assert isinstance(slice_, (slice, int)), "slice_ has to be either None or a int/slice object"
+            pins = p.pins[slice_]
+            sig = myhdl.Signal(p.sig[slice_])
+        kws = p.kwargs.copy()
+        kws.update(kwargs)
+        self.add_port(name, sig, pins, **kws)
             
     def run(self):
         pass
