@@ -66,10 +66,12 @@ class _ise(object):
         if isinstance(fn, (list,tuple)):
             for ff in fn:
                 if not isinstance(ff,str):
-                    raise TYpeError("list or tuple entry needs ot be a string")
-                self.hdl_file_list.append(ff)
+                    raise TYpeError("list or tuple entry needs to be a string")
+                if ff not in self.hdl_file_list:
+                    self.hdl_file_list.append(ff)
         elif isinstance(files,str):
-            self.hdl_file_list.append(files)
+            if ff not in self.hdl_file_list:
+                self.hdl_file_list.append(files)
         
     def create_tcl(self, filename=None):
         """Create the project TCL script
@@ -123,10 +125,16 @@ class _ise(object):
         # Create or open an ISE project (xise?)
         print('Project name : %s ' % (pj_fn))
         pjfull = os.path.join(self._path,pj_fn)
+
+        # let the TCL file be the master file, always create
+        # a new project.  If a user uses this to "bootstrap"
+        # the need to take care to rename the project if modfied
+        # else it will be overwritten.
         if os.path.isfile(pjfull):
-            self.tcl_script += 'project open %s \n' % (pj_fn)
-        else:
-            self.tcl_script += 'project new %s\n' % (pj_fn)
+            os.remove(pjfull)
+        #    self.tcl_script += 'project open %s \n' % (pj_fn)
+        #else:
+        self.tcl_script += 'project new %s\n' % (pj_fn)
     
         if self.fpga.family:
             self.tcl_script += 'project set family %s\n'%self.fpga.family
@@ -137,7 +145,7 @@ class _ise(object):
         # add the hdl files
         self.tcl_script += '\n'
         self.tcl_script += '# add hdl files:\n'
-        #self.tcl_script += 'xfile add %s\n'%(ucffn)
+        self.tcl_script += 'xfile add %s\n'%(ucffn)
         for hdl_file in self.hdl_file_list:
             self.tcl_script += 'xfile add %s\n'%hdl_file
  
@@ -168,7 +176,7 @@ class _ise(object):
         fid = open(fn, 'w')
         fid.write(self.tcl_script)
         fid.close() 
- 
+
     def run(self, filename=None):
         """Run the created TCL script """
         if filename:
@@ -186,6 +194,7 @@ class _ise(object):
                                         stderr=subprocess.STDOUT)
         except Exception, err:
             print(err)
+            raise err
 
     @property
     def fpga(self):
@@ -286,19 +295,39 @@ class XilinxFPGA(_fpga):
         print(ustr)
 
     def run(self, use='verilog'):
-        self.convert(to=use)
+        """Create the files and run the FPGA toolflow
+        This function will convert the MyHDL design, create the 
+        constraint files, and run the FPGA toolflow.
+        """
+        print(self.top_name)
+        use = use.lower()
+        ext = {'verilog':'.v', 'vhdl':'vhd'}
+        
+        # convert the design to the intermidiate V* 
+        self._scrub(self.hdl_file_list)
+        cfiles = self.convert(to=use)
+        for ff in cfiles:            
+            self.hdl_file_list.append(ff)
+        
+        # create the constraints and project files
         self.create_ucf(filename=self.name+'.ucf')
         self._tool_chain = _ise(top_name=self.top_name,path=self._path)
         self._tool_chain.fpga = self
         pprint(vars(self._tool_chain.fpga))
-
-        # @todo: verify the following is correct, that is the filename
-        #        doesnt' already exist some where.
-        self.hdl_file_list.append(self.top_name+'.v')
+        
+        # add the files to the toolchain
         self._tool_chain.add_files(self.hdl_file_list)
 
         # create the TCL file and run the tools
         self._tool_chain.create_tcl()
+        #self._tool_chain.run()
+
+    def rerun(self):
+        """Run the FPGA toolflow using the existing files
+        """
+        self._tool_chain = _ise(top_name=self.top_name,path=self._path)
+        self._tool_chain.fpga = self
+        pprint(vars(self._tool_chain.fpga))
         self._tool_chain.run()
         
     def __repr__(self):
