@@ -10,3 +10,6 @@ from _xupv2p import xupv2p
 
 from _ufo400 import ufo400
 from _sx1 import sx1
+
+# Papilio boards
+from _papilio import pone
\ No newline at end of file
